package config

import (
	"fmt"
	"strings"

	"github.com/spf13/viper"
	"github.com/wavefronthq/wavefront-sdk-go/application"
)

const (
	// ===== CONFIG KEYS ===== //.

	AppNameKey         = "app.name"
	AppShutdownWaitKey = "app.shutdown.wait"
	AppEnvKey          = "env"
	AppRegionKey       = "region"
	AppDryRunKey       = "app.dryrun"

	CbdClientEnabledKey         = "cbd.client.enabled"
	CbdClientStatusURI          = "cbd.client.status.uri"
	CbdClientMetadataURI        = "cbd.client.metadata.uri"
	CbdClientRequestTimeoutKey  = "cbd.client.request.timeout"
	CbdClientMaxIdleConnsKey    = "cbd.client.idle.max"
	CbdClientMaxConnsKey        = "cbd.client.conn.max"
	CbdClientIdleConnTimeoutKey = "cbd.client.idle.timeout"

	DeviceAuthClientEnabled            = "deviceauth.client.enabled"
	DeviceAuthClientRequestTimeoutKey  = "deviceauth.client.request.timeout"
	DeviceAuthClientMaxIdleConnsKey    = "deviceauth.client.idle.max"
	DeviceAuthClientMaxConnsKey        = "deviceauth.client.conn.max"
	DeviceAuthClientIdleConnTimeoutKey = "deviceauth.client.idle.timeout"
	DeviceAuthHostKey                  = "deviceauth.host"
	DeviceAuthPortKey                  = "deviceauth.port"

	DeviceEventsEnabledKey = "device.events.enabled"

	DBEnvKey                      = "pgenv"
	DBHostKey                     = "pghost"
	DBHostMultiDBKey              = "pghost.multi.db" // pragma: allowlist secret
	DBHostReaderKey               = "pghost.reader"
	DBUserKey                     = "pguser"
	DBPassKey                     = "pgpassword" // pragma: allowlist secret
	DBDatabaseKey                 = "pgdatabase" // pragma: allowlist secret
	DBPortKey                     = "pgport"
	DBDriverKey                   = "db.driver"
	DBEndpointOverrideKey         = "pg.endpoint.override"
	DBSimulateLoadEnabledKey      = "pg.simulate.load"
	DBSimulateLoadTrafficRatioKey = "pg.simulate.load.traffic"
	DBSSLModeKey                  = "pgsslmode"

	DevModeKey = "dev"

	DeviceTableNameKey        = "device.table.name"
	DynamoDBEndpointKey       = "dynamodb.endpoint"
	DynamoDBMaxAttemptsKey    = "dynamodb.max.attempts"
	DynamoDBMaxBackoffKey     = "dynamodb.max.backoff"
	DynamoDBRequestTimeoutKey = "dynamodb.request.timeout"

	ElasticsearchEndpointKey = "elasticsearch.endpoint"
	ElasticsearchUsernameKey = "elasticsearch.username"
	ElasticsearchPasswordKey = "elasticsearch.password" // pragma: allowlist secret

	ElasticsearchMultiConfigKey  = "elasticsearch.multiconfig"
	ElasticsearchInsecureEnvsKey = "elasticsearch.insecure_envs"

	EvictDelayEnabledKey      = "evict.delay.enabled"
	EvictDelayMaxBatchSizeKey = "evict.delay.max.batch.size"
	EvictDelayTimeKey         = "evict.delay.time"

	GrpcPortKey = "grpc.port"

	HTTPReadHeaderTimeoutKey = "http.read.header.timeout"
	HTTPReadTimeoutKey       = "http.read.timeout"
	HTTPRequestIDHeaderKey   = "http.request.id.header"
	HTTPRequestTimeoutKey    = "http.request.timeout"

	KinesisEndpointKey               = "kinesis.endpoint"
	KinesisMaxAttemptsKey            = "kinesis.max.attempts"
	KinesisMaxBackoffKey             = "kinesis.max.backoff"
	KinesisRequestTimeoutKey         = "kinesis.request.timeout"
	KinesisStreamNameDeviceEventsKey = "kinesis.stream.name.device_events"

	LogLevelKey              = "log.level"
	LogCallerKey             = "log.caller"
	LogFormatKey             = "log.format"
	LogSamplerBurstKey       = "log.sampler.burst"
	LogSamplerPeriodSecKey   = "log.sampler.period"
	LogSamplerNextSamplerKey = "log.sampler.next.sampler"
	LogStdlibKey             = "log.stdlib"

	MetricsEnabledKey        = "metrics.enabled"
	MetricsPrefixKey         = "metrics.prefix"
	MetricsRuntimeEnabledKey = "metrics.runtime.enabled"
	TracingEnabledKey        = "tracing.enabled"

	WebPortKey         = "web.port"
	WebRoutesPrefixKey = "web.routes.prefix"

	WFModeKey              = "wavefront.mode"
	WFHostKey              = "wavefront.host"
	WFTokenKey             = "wavefront.token"
	WFSamplerRateKey       = "wavefront.sampler_rate"
	WFBatchSizeKey         = "wavefront.batch_size"
	WFMaxBufferSizeKey     = "wavefront.max_buffer_size"
	WFFlushIntervalSecsKey = "wavefront.flush_interval_secs"
	WFPortsMetricsKey      = "wavefront.ports.metrics"
	WFPortsDistributionKey = "wavefront.ports.distribution"
	WFPortsTracingKey      = "wavefront.ports.tracing"
	WFPortsEventsKey       = "wavefront.ports.events"
	WFTagsApplicationKey   = "wavefront.tags.application"
	WFTagsClusterKey       = "wavefront.tags.cluster"
	WFTagsShardKey         = "wavefront.tags.shard"
	WFTagsCustomKey        = "wavefront.tags.custom"

	// ===== DEFAULTS ===== //.

	defaultAppEnv          = "local"
	defaultAppShutdownWait = "5s"
	defaultGrpcPort        = 5004

	defaultDBDriver               = "postgres"
	defaultDBSSLMode              = "disable"
	defaultDBEnv                  = "local"
	defaultDBSimulateLoadEnabled  = false
	defaultDBSimulateTrafficRatio = 0

	defaultCbdClientEnabled         = false
	defaultCbdClientRequestTimeout  = "1s"
	defaultCbdClientMaxIdleConns    = 2
	defaultCbdClientMaxConns        = 2
	defaultCbdClientIdleConnTimeout = "600s"
	defaultCbdClientStatusURI       = "/services/emd_status/"
	defaultCbdClientMetadataURI     = "/services/emd_metadata/"

	defaultDeviceAuthHost                  = "http://ems-device-auth"
	defaultDeviceAuthPort                  = 5004
	defaultDeviceAuthClientRequestTimeout  = "1s"
	defaultDeviceAuthClientMaxIdleConns    = 2
	defaultDeviceAuthClientMaxConns        = 2
	defaultDeviceAuthClientIdleConnTimeout = "600s"

	defaultDynamoDBMaxAttempts    = 3
	defaultDynamoDBMaxBackoffTime = "5s"
	defaultDynamoDBRequestTimeout = "5s"

	defaultEvictDelayEnabled      = false
	defaultEvictDelayMaxBatchSize = 100
	defaultEvictDelayTime         = "10s"

	defaultHTTPReadHeaderTimeout = "1s"
	defaultHTTPReadTimeout       = "1s"
	defaultHTTPRequestTimeout    = "100ms"

	defaultKinesisMaxAttempts    = 3
	defaultKinesisMaxBackoffTime = "5s"
	defaultKinesisRequestTimeout = "1s"

	defaultLogSamplerBurst      = 20
	defaultLogSamplerPeriod     = "3s"
	defaultLogSamplerNextPeriod = 100

	defaultMetricsPrefix = "psc.ems.duc"

	defaultWebPort = 8080

	defaultWFFlushIntervalSecs = 1
	defaultWFHost              = "wavefront-proxy"
	defaultWFMetricsPort       = 2878
	defaultWFMode              = "proxy"
	defaultWFTagsApplication   = "psc"
	defaultWFTracingPort       = 30000
	//nolint:lll
	defaultElasticsearchInsecureEnvs = "local dev01 kcdev01 test03 kctest03 eap01 prod01 prod02 prod05 prd5use1 prod06 prd1ec1 prodsyd prd1apse2 prodnrt prd1apne1"
	defaultDeviceEventsEnabled       = true

	// ===== Headers ===== //.

	// defaultRequestIDHeader is set to use the WaveFront OpenTracing TraceID.
	defaultRequestIDHeader = "Wf-Ot-Traceid"
	// RequestContextHeader is the PSC specific request data header key.
	RequestContextHeader = "X-Request-Context"
	// UserContextHeader is the PSC specific user authentication header key.
	UserContextHeader = "X-User-Context"
	// ConnectorAuthHeader is the HTTP header for passing connector keys.
	ConnectorAuthHeader = "X-Auth-Token"

	// ===== OTHER ===== //.

	maxSamplerRate  uint64 = 100
	HealthCheckPath string = "/healthCheck"
)

func Init() error {
	viper.SetEnvKeyReplacer(strings.NewReplacer(".", "_", "-", "_"))
	viper.AutomaticEnv()
	setDefaults()

	err := load()

	viper.SetDefault(WebRoutesPrefixKey, fmt.Sprintf("/%s", viper.GetString(AppNameKey)))

	return err
}

func GetWFAppTags() application.Tags {
	appTags := application.New(viper.GetString(WFTagsApplicationKey), viper.GetString(AppNameKey))

	if viper.IsSet(WFTagsClusterKey) {
		appTags.Cluster = viper.GetString(WFTagsClusterKey)
	}

	if viper.IsSet(WFTagsShardKey) {
		appTags.Shard = viper.GetString(WFTagsShardKey)
	}

	if customTags := viper.GetStringMapString(WFTagsCustomKey); customTags != nil {
		appTags.CustomTags = customTags
	}

	return appTags
}

func GetWFSamplerRate() uint64 {
	if samplerRate := viper.GetUint64(WFSamplerRateKey); samplerRate < maxSamplerRate {
		return samplerRate
	}

	return 0
}

func IsRedactedHeader(header string) bool {
	return map[string]bool{
		RequestContextHeader: true,
		UserContextHeader:    true,
		ConnectorAuthHeader:  true,
	}[header]
}

// SetDefaultConfig sets default values for config.
// nolint: funlen
func setDefaults() {
	viper.SetDefault(AppEnvKey, defaultAppEnv)
	viper.SetDefault(AppShutdownWaitKey, defaultAppShutdownWait)
	viper.SetDefault(DeviceAuthHostKey, defaultDeviceAuthHost)
	viper.SetDefault(DeviceAuthPortKey, defaultDeviceAuthPort)
	viper.SetDefault(DeviceAuthClientEnabled, false)
	viper.SetDefault(DeviceAuthClientRequestTimeoutKey, defaultDeviceAuthClientRequestTimeout)
	viper.SetDefault(DeviceAuthClientMaxConnsKey, defaultDeviceAuthClientMaxConns)
	viper.SetDefault(DeviceAuthClientMaxIdleConnsKey, defaultDeviceAuthClientMaxIdleConns)
	viper.SetDefault(DeviceAuthClientIdleConnTimeoutKey, defaultDeviceAuthClientIdleConnTimeout)
	viper.SetDefault(DeviceEventsEnabledKey, defaultDeviceEventsEnabled)
	viper.SetDefault(DynamoDBMaxAttemptsKey, defaultDynamoDBMaxAttempts)
	viper.SetDefault(DynamoDBMaxBackoffKey, defaultDynamoDBMaxBackoffTime)
	viper.SetDefault(DynamoDBRequestTimeoutKey, defaultDynamoDBRequestTimeout)
	viper.SetDefault(EvictDelayEnabledKey, defaultEvictDelayEnabled)
	viper.SetDefault(EvictDelayMaxBatchSizeKey, defaultEvictDelayMaxBatchSize)
	viper.SetDefault(EvictDelayTimeKey, defaultEvictDelayTime)
	viper.SetDefault(HTTPReadHeaderTimeoutKey, defaultHTTPReadHeaderTimeout)
	viper.SetDefault(HTTPReadTimeoutKey, defaultHTTPReadTimeout)
	viper.SetDefault(HTTPRequestIDHeaderKey, defaultRequestIDHeader)
	viper.SetDefault(HTTPRequestTimeoutKey, defaultHTTPRequestTimeout)
	viper.SetDefault(KinesisMaxAttemptsKey, defaultKinesisMaxAttempts)
	viper.SetDefault(KinesisMaxBackoffKey, defaultKinesisMaxBackoffTime)
	viper.SetDefault(KinesisRequestTimeoutKey, defaultKinesisRequestTimeout)
	viper.SetDefault(MetricsPrefixKey, defaultMetricsPrefix)
	viper.SetDefault(WebPortKey, defaultWebPort)
	viper.SetDefault(GrpcPortKey, defaultGrpcPort)
	viper.SetDefault(ElasticsearchInsecureEnvsKey, defaultElasticsearchInsecureEnvs)
	viper.SetDefault(CbdClientEnabledKey, defaultCbdClientEnabled)
	viper.SetDefault(CbdClientRequestTimeoutKey, defaultCbdClientRequestTimeout)
	viper.SetDefault(CbdClientMaxConnsKey, defaultCbdClientMaxConns)
	viper.SetDefault(CbdClientMaxIdleConnsKey, defaultCbdClientMaxIdleConns)
	viper.SetDefault(CbdClientIdleConnTimeoutKey, defaultCbdClientIdleConnTimeout)
	viper.SetDefault(CbdClientStatusURI, defaultCbdClientStatusURI)
	viper.SetDefault(CbdClientMetadataURI, defaultCbdClientMetadataURI)

	if viper.GetBool(DevModeKey) {
		viper.SetDefault(LogLevelKey, "info")
		viper.SetDefault(LogFormatKey, "pretty")
		viper.SetDefault(LogCallerKey, true)
	} else {
		viper.SetDefault(LogLevelKey, "warn")
	}

	viper.SetDefault(LogSamplerBurstKey, defaultLogSamplerBurst)
	viper.SetDefault(LogSamplerPeriodSecKey, defaultLogSamplerPeriod)
	viper.SetDefault(LogSamplerNextSamplerKey, defaultLogSamplerNextPeriod)

	viper.SetDefault(DBEnvKey, defaultDBEnv)
	viper.SetDefault(DBSSLModeKey, defaultDBSSLMode)
	viper.SetDefault(DBDriverKey, defaultDBDriver)
	viper.SetDefault(DBSimulateLoadEnabledKey, defaultDBSimulateLoadEnabled)
	viper.SetDefault(DBSimulateLoadTrafficRatioKey, defaultDBSimulateTrafficRatio)
	viper.SetDefault(MetricsEnabledKey, true)
	viper.SetDefault(TracingEnabledKey, false)
	viper.SetDefault(WFModeKey, defaultWFMode)
	viper.SetDefault(WFHostKey, defaultWFHost)
	viper.SetDefault(WFPortsMetricsKey, defaultWFMetricsPort)
	viper.SetDefault(WFPortsTracingKey, defaultWFTracingPort)
	viper.SetDefault(WFFlushIntervalSecsKey, defaultWFFlushIntervalSecs)
	viper.SetDefault(WFTagsApplicationKey, defaultWFTagsApplication)
	viper.SetDefault(WFTagsClusterKey, viper.GetString(AppRegionKey))
	viper.SetDefault(WFTagsShardKey, viper.GetString("env"))
}

func load() error {
	viper.SetConfigName("config")
	viper.SetConfigType("yaml")
	viper.AddConfigPath("/config")
	viper.AddConfigPath(".")

	if err := viper.ReadInConfig(); err != nil {
		return fmt.Errorf("failed to load config: %w", err)
	}

	return nil
}
