package controller

import (
	"fmt"
	"net/http"

	"github.com/rs/zerolog/log"

	"gitlab.bit9.local/psc/platform/ems/device-service/internal/service"

	"github.com/labstack/echo/v4"
)

func CreateHealthCheck(svc service.ReadinessChecker) func(e echo.Context) error {
	return func(ctx echo.Context) error {
		status := http.StatusNoContent

		// determine if app is ready to receive requests based on
		// healthy connections to down stream dependencies
		if ctx.QueryParam("isReady") == "true" {
			if err := svc.IsReady(); err != nil {
				status = http.StatusServiceUnavailable

				log.Error().Err(err).Msg("service is not ready")
			}
		}

		if err := ctx.NoContent(status); err != nil {
			return fmt.Errorf("failed to render health check: %w", err)
		}

		return nil
	}
}
