package controller_test

import (
	"net/http"
	"net/url"

	"gitlab.bit9.local/psc/platform/ems/device-service/internal/config"
	"gitlab.bit9.local/psc/platform/ems/device-service/internal/controller"
	"gitlab.bit9.local/psc/platform/ems/device-service/internal/service"
	"gitlab.bit9.local/psc/platform/ems/device-service/test/mocks"

	"github.com/labstack/echo/v4"
	. "github.com/onsi/ginkgo"
	"github.com/spf13/viper"
)

var _ = Describe("HealthCheck", func() {
	const prefix = "/test"
	var testEcho *echo.Echo
	var mockReadinessChecker *mocks.ReadinessChecker
	var mockPerformer *mocks.Performer

	newRequest := func() *http.Request {
		//nolint:exhaustruct // only used for testing, just need method and url
		return &http.Request{
			Method: http.MethodGet,
			URL:    &url.URL{Path: "/test/healthCheck"},
		}
	}

	BeforeEach(func() {
		testEcho = echo.New()
		mockPerformer = mocks.NewPerformer(ctrl)
		mockReadinessChecker = mocks.NewReadinessChecker(ctrl)

		viper.Set(config.WebRoutesPrefixKey, prefix)
		controller.Route(testEcho, mockPerformer, mockReadinessChecker)
	})

	Context("health check api", func() {
		var mockResponseWriter *mocks.ResponseWriter

		BeforeEach(func() {
			mockResponseWriter = mocks.NewResponseWriter(ctrl)
		})

		It("should report http 204 when live", func() {
			mockResponseWriter.EXPECT().WriteHeader(http.StatusNoContent).Times(1)

			testEcho.ServeHTTP(mockResponseWriter, newRequest())
		})

		It("should report http 204 when ready", func() {
			request := newRequest()
			q := request.URL.Query()
			q.Add("isReady", "true")
			request.URL.RawQuery = q.Encode()

			mockReadinessChecker.EXPECT().IsReady().Times(1)
			mockResponseWriter.EXPECT().WriteHeader(http.StatusNoContent).Times(1)

			testEcho.ServeHTTP(mockResponseWriter, request)
		})

		It("should report http 503 when readiness check fails", func() {
			request := newRequest()
			q := request.URL.Query()
			q.Add("isReady", "true")
			request.URL.RawQuery = q.Encode()

			mockReadinessChecker.EXPECT().IsReady().Return(service.ErrAppNotReady)
			mockResponseWriter.EXPECT().WriteHeader(http.StatusServiceUnavailable).Times(1)

			testEcho.ServeHTTP(mockResponseWriter, request)
		})
	})
})
