package v1_test

import (
	"errors"
	"net/http"

	v1 "gitlab.bit9.local/psc/platform/ems/device-service/internal/controller/v1"
	"gitlab.bit9.local/psc/platform/ems/device-service/test/mocks"

	"github.com/golang/mock/gomock"
	"github.com/labstack/echo/v4"
	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
	"github.com/rs/zerolog"
)

var (
	ErrFailure              = errors.New("failure")
	ErrMockedServiceFailure = errors.New("mocked service failure")
)

var _ = Describe("root", func() {
	var ctrl *gomock.Controller
	var mockContext *mocks.EchoContext
	var mockPerformer *mocks.Performer
	var testEcho *echo.Echo

	BeforeSuite(func() {
		ctrl = gomock.NewController(GinkgoT())
		mockContext = mocks.NewEchoContext(ctrl)
		mockPerformer = mocks.NewPerformer(ctrl)
		testEcho = echo.New()
	})

	Context("handler", func() {
		var mockResponseWriter *mocks.ResponseWriter
		var responseHeaders http.Header
		var request *http.Request
		var requestLogger *zerolog.Logger

		BeforeEach(func() {
			//nolint:exhaustruct // only used for testing, just need empty/default request
			request = &http.Request{}
			requestLogger = zerolog.Ctx(request.Context())
			responseHeaders = http.Header{}
			mockResponseWriter = mocks.NewResponseWriter(ctrl)

			mockResponseWriter.EXPECT().Header().Return(responseHeaders).Times(1)
		})

		It("should succeed when service succeeds", func() {
			mockResponseWriter.EXPECT().WriteHeader(http.StatusAccepted).Times(1)
			mockPerformer.EXPECT().Perform(requestLogger).Times(1)
			echoCtx := testEcho.NewContext(request, mockResponseWriter)
			err := v1.CreatePOSTRootHandler(mockPerformer)(echoCtx)
			Expect(err).To(BeNil())
		})

		It("should fail when service fails", func() {
			mockResponseWriter.EXPECT().WriteHeader(http.StatusInternalServerError).Times(1)
			mockResponseWriter.EXPECT().Write(gomock.Any()).Times(1)
			mockPerformer.EXPECT().Perform(requestLogger).Return(ErrMockedServiceFailure).Times(1)
			echoCtx := testEcho.NewContext(request, mockResponseWriter)
			err := v1.CreatePOSTRootHandler(mockPerformer)(echoCtx)
			Expect(err).ToNot(BeNil())
		})

		It("should fail when render fails", func() {
			mockPerformer.EXPECT().Perform(requestLogger).Times(1)
			mockContext.EXPECT().NoContent(http.StatusAccepted).Return(ErrFailure).Times(1)
			mockContext.EXPECT().Request().Return(request).Times(1)
			err := v1.CreatePOSTRootHandler(mockPerformer)(mockContext)
			Expect(err).ToNot(BeNil())
		})
	})
})
