package data_test

import (
	"context"
	"path/filepath"
	"testing"

	"github.com/golang/mock/gomock"
	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
	"github.com/rs/zerolog/log"
	"github.com/testcontainers/testcontainers-go"
	"github.com/testcontainers/testcontainers-go/modules/postgres"
	"github.com/testcontainers/testcontainers-go/wait"
)

var (
	ctrl      *gomock.Controller          //nolint:gochecknoglobals // singleton test controller
	container *postgres.PostgresContainer //nolint:gochecknoglobals // singleton test postgres container
	pgCtx     context.Context             //nolint:gochecknoglobals // singleton test postgres context
)

func TestData(t *testing.T) {
	ctrl = gomock.NewController(t)
	t.Parallel()
	RegisterFailHandler(Fail)
	RunSpecs(t, "Marshal")
}

var _ = Describe("data package", func() {
	BeforeSuite(func() {
		pgCtx = context.Background()

		var err error
		container, err = postgres.RunContainer(pgCtx,
			postgres.WithDatabase("database"),
			postgres.WithUsername("user"),
			postgres.WithPassword("password"),
			postgres.WithInitScripts(filepath.Join("../../test", "init-db.sql")),
			testcontainers.WithWaitStrategy(wait.ForListeningPort("5432")),
		)
		if err != nil {
			log.Panic().Err(err).Msg("error running container")
		}

		container.FollowOutput(&logConsumer{})
	})

	AfterSuite(func() {
		if container == nil {
			return
		}

		if err := container.Terminate(pgCtx); err != nil {
			log.Info().Err(err).Msg("Failed to stop postgres container in test")
		}
	})
})
