package data

// CBD device-services update device SQL:
// UPDATE device
// SET
// 	device_type_id = ?,    // inDevice.getDeviceType().getCode()
// 	device_owner_id = ?,   // inDevice.getDeviceOwnerId()
// 	device_guid = ?,       // inDevice.getDeviceGuid()
// 	name = ?,              // inDevice.getName()
// 	email = ?,             // inDevice.getEmail()
// 	email_type_id = ?,     // Integer(inDevice.getEmailType().getCode())
// 	session_id = ?,        // inDevice.getDeviceSessionId()
// 	properties = ?,        // mObjectMapper.writeValueAsString(inDevice.getProperties())
// 	assignedToId = ?,      // inDevice.getAssignedToId()
// 	assignedToName = ?,    // inDevice.getAssignedToName()
// 	change_timestamps = ?, // mObjectMapper.writeValueAsString(inDevice.getChangeTimestamps())
// 	time_correction = ?,   // inDevice.getTimeCorrectionMs()
// 	replica_device_id = ?, // inDevice.getReplicaDeviceId()
// 	kill_switch_on = ?,    // inDevice.isKillSwitchOn()
// 	org_key = ?,           // inDevice.getOrgKey()
// 	deployment_type = ?    // inDevice.getDeploymetType().name()
// 	update_version = update_version + 1,
// WHERE
// 	device_id = ? AND      // inDevice.getId()
// 	update_version = ?     // inDevice.getUpdateVersion()

const (
	sqlUpdateDevice = `
WITH d_props_json AS (
  SELECT COALESCE(CAST(properties as jsonb), '{}') || $7 AS props
  FROM device
  WHERE device_id = $1
)
UPDATE device SET
  device_type_id = $2,
  name = $3,
  email = $4,
  deployment_type = $5,
  org_key = $6,
  properties = (SELECT props FROM d_props_json),
  update_version = update_version + 1
WHERE device_id = $1 AND EXISTS
(
SELECT
  device.device_type_id,
  device.name,
  device.email,
  device.deployment_type,
  device.org_key,
  CAST(device.properties as jsonb)
EXCEPT
SELECT
  $2,
  $3,
  $4,
  $5,
  $6,
  (SELECT props FROM d_props_json)
);`

	sqlInsertDeviceToMSM = `INSERT INTO msm_processing_device(device_id) VALUES ($1) ON CONFLICT DO NOTHING;`

	sqlFindDevice = `SELECT * FROM device WHERE device_id = %d;`
)
