package data_test

import (
	"context"
	"errors"

	"carbonblack.com/protocols/psc/ems/server/emdstatus"
	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
	"gitlab.bit9.local/psc/platform/ems/device-service/internal/data"
	"gitlab.bit9.local/psc/platform/ems/device-service/test/mocks"
)

var errDB = errors.New("something happened in the db")

//nolint:exhaustruct
var _ = Describe("Table", func() {
	var mockDBClient *mocks.DBClient
	var table *data.Table
	partitionKey1 := "RegistrationID"
	partitionKey2 := "DeviceGUID"
	sortKey := "LastUpdateTime"
	devTableSpecs := data.DeviceTableSpecs()

	Context("initialized", func() {
		BeforeEach(func() {
			mockDBClient = mocks.NewDBClient(ctrl)
			mockDBClient.EXPECT().Ready(devTableSpecs.TableName).Return(nil).Times(1)
			table, _ = data.NewTable(mockDBClient, devTableSpecs)
			Expect(table).ToNot(BeNil())
		})

		Context("Update method", func() {
			It("should not fail with message containing composite hash key", func() {
				msg := &emdstatus.DeviceStatusMsg{
					Metadata: map[string]string{partitionKey1: "val1", partitionKey2: "val2", sortKey: "val3"},
				}
				mockDBClient.EXPECT().Update(context.TODO(), devTableSpecs.TableName, msg).Return(nil)
				err := table.Update(context.TODO(), msg)
				Expect(err).To(BeNil())
			})

			It("should fail with message missing composite key", func() {
				msg := &emdstatus.DeviceStatusMsg{
					Metadata: map[string]string{sortKey: "val3"},
				}
				err := table.Update(context.TODO(), msg)
				Expect(err).ToNot(BeNil())
			})

			It("should fail with message missing range key", func() {
				msg := &emdstatus.DeviceStatusMsg{
					Metadata: map[string]string{partitionKey1: "val1", partitionKey2: "val2"},
				}
				err := table.Update(context.TODO(), msg)
				Expect(err).ToNot(BeNil())
			})
		})
	})

	Context("failed initialization", func() {
		It("should return error when dynamodb table cannot be reached", func() {
			mockDBClient = mocks.NewDBClient(ctrl)
			mockDBClient.EXPECT().Ready(devTableSpecs.TableName).Return(errDB).Times(1)
			_, err := data.NewTable(mockDBClient, devTableSpecs)
			Expect(err).ToNot(BeNil())
		})
	})
})
