//nolint:nlreturn,wrapcheck,testpackage // test file, brevity, testing internal helper funcs
package model

import (
	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
)

var _ = Describe("DB Model", func() {
	Context("with bad data", func() {
		It("should fail to create json properties", func() {
			badData1 := map[string]string{
				DataFieldOSMajorVersion:     "",
				DataFieldOSMinorVersion:     "",
				DataFieldMaintenanceRelease: "",
				DataFieldOSBuildNumber:      "",
				DataFieldPlatformType:       "",
			}
			badData2 := map[string]string{
				DataFieldOSMajorVersion:     "abc",
				DataFieldOSMinorVersion:     "abc",
				DataFieldMaintenanceRelease: "abc",
				DataFieldOSBuildNumber:      "abc",
				DataFieldPlatformType:       "abc",
			}
			badData3 := map[string]string{
				DataFieldOSMajorVersion:     "",
				DataFieldOSMinorVersion:     "5",
				DataFieldMaintenanceRelease: "5",
				DataFieldOSBuildNumber:      "5",
				DataFieldPlatformType:       "5",
			}
			badData4 := map[string]string{
				DataFieldOSMajorVersion:     "abc",
				DataFieldOSMinorVersion:     "5",
				DataFieldMaintenanceRelease: "5",
				DataFieldOSBuildNumber:      "5",
				DataFieldPlatformType:       "5",
			}
			badData5 := map[string]string{
				DataFieldOSMajorVersion:     "2320",
				DataFieldOSMinorVersion:     "",
				DataFieldMaintenanceRelease: "5",
				DataFieldOSBuildNumber:      "5",
				DataFieldPlatformType:       "2",
			}
			badData6 := map[string]string{
				DataFieldOSMajorVersion:     "2320",
				DataFieldOSMinorVersion:     "abc",
				DataFieldMaintenanceRelease: "5",
				DataFieldOSBuildNumber:      "5",
				DataFieldPlatformType:       "2",
			}
			badData7 := map[string]string{
				DataFieldOSMajorVersion:     "5",
				DataFieldOSMinorVersion:     "5",
				DataFieldMaintenanceRelease: "5",
				DataFieldOSBuildNumber:      "",
				DataFieldPlatformType:       "5",
			}
			badData8 := map[string]string{
				DataFieldOSMajorVersion:     "5",
				DataFieldOSMinorVersion:     "5",
				DataFieldMaintenanceRelease: "5",
				DataFieldOSBuildNumber:      "abc",
				DataFieldPlatformType:       "5",
			}
			badData9 := map[string]string{
				DataFieldOSMajorVersion:     "5",
				DataFieldOSMinorVersion:     "5",
				DataFieldMaintenanceRelease: "5",
				DataFieldOSBuildNumber:      "5",
				DataFieldPlatformType:       "",
			}
			badData10 := map[string]string{
				DataFieldOSMajorVersion:     "5",
				DataFieldOSMinorVersion:     "5",
				DataFieldMaintenanceRelease: "5",
				DataFieldOSBuildNumber:      "5",
				DataFieldPlatformType:       "abc",
			}

			Expect(func() error { _, err := propertiesJSON(map[string]string{}); return err }()).ShouldNot(Succeed())
			Expect(func() error { _, err := propertiesJSON(badData1); return err }()).ShouldNot(Succeed())
			Expect(func() error { _, err := propertiesJSON(badData2); return err }()).ShouldNot(Succeed())
			Expect(func() error { _, err := propertiesJSON(badData3); return err }()).ShouldNot(Succeed())
			Expect(func() error { _, err := propertiesJSON(badData4); return err }()).ShouldNot(Succeed())
			Expect(func() error { _, err := propertiesJSON(badData5); return err }()).ShouldNot(Succeed())
			Expect(func() error { _, err := propertiesJSON(badData6); return err }()).ShouldNot(Succeed())
			Expect(func() error { _, err := propertiesJSON(badData7); return err }()).Should(Succeed())
			Expect(func() error { _, err := propertiesJSON(badData8); return err }()).Should(Succeed())
			Expect(func() error { _, err := propertiesJSON(badData9); return err }()).ShouldNot(Succeed())
			Expect(func() error { _, err := propertiesJSON(badData10); return err }()).ShouldNot(Succeed())
		})
	})

	It("should create json properties", func() {
		data1 := map[string]string{
			DataFieldOSMajorVersion:     "2320",
			DataFieldOSMinorVersion:     "5",
			DataFieldMaintenanceRelease: "5",
			DataFieldOSBuildNumber:      "5",
			DataFieldPlatformType:       "2",
		}

		Expect(propertiesJSON(data1)).To(Equal(
			//nolint:lll // test file, need to check for a long string
			"{\"Maintenance Release\":\"5\",\"Build Number\":\"5\",\"OS Major Version\":\"Ubuntu 20\",\"OS Minor Version\":\"5\",\"Platform Type\":\"2\",\"_SensorKitType\":\"UBUNTU\",\"_VersionInfo\":\"Ubuntu 20.05.5 x64\"}",
		))
	})
})
