package model

import "strings"

type (
	DeploymentTypeEnum         int
	SubDeploymentTypeEnum      int
	InfrastructureProviderEnum int
	VirtualizationProviderEnum int
	VDIProviderEnum            int
)

const (
	DeploymentTypeEndpoint DeploymentTypeEnum = iota // 0
	DeploymentTypeWorkload                           // 1
	DeploymentTypeVDI                                // 2
	DeploymentTypeAWS                                // 3
	DeploymentTypeAzure                              // 4
	DeploymentTypeGCP                                // 5
)

const (
	SubDeploymentTypeNone                 SubDeploymentTypeEnum = iota // 0
	SubDeploymentTypeVMwareVirtualMachine                              // 1
	SubDeploymentTypeAWSVirtualMachineEC2                              // 2
)

const (
	InfrastructureProviderNone  InfrastructureProviderEnum = iota // 0
	InfrastructureProviderAWS                                     // 1
	InfrastructureProviderAzure                                   // 2
	InfrastructureProviderGCP                                     // 3
)

const (
	VirtualizationProviderNone     VirtualizationProviderEnum = iota // 0
	VirtualizationProviderWorkload                                   // 1
	VirtualizationProviderAWS                                        // 2
)

const (
	VDIProviderNone    VDIProviderEnum = iota // 0
	VDIProviderHorizon                        // 1
	VDIProviderCitrix                         // 2
)

func (dte DeploymentTypeEnum) String() string {
	return []string{
		"ENDPOINT",
		"WORKLOAD",
		"VDI",
		"AWS",
		"AZURE",
		"GCP",
	}[dte]
}

func (sdte SubDeploymentTypeEnum) String() string {
	return []string{
		"NONE",
		"VMWARE_VIRTUAL_MACHINE",
		"AWS_VIRTUAL_MACHINE_EC2",
	}[sdte]
}

func (ipe InfrastructureProviderEnum) String() string {
	return []string{
		"NONE",
		"AWS",
		"AZURE",
		"GCP",
	}[ipe]
}

func (vpe VirtualizationProviderEnum) String() string {
	return []string{
		"NONE",
		"VMW_ESX",
		"AWS-EC2",
	}[vpe]
}

func (vdipe VDIProviderEnum) String() string {
	return []string{
		"NONE",
		"HORIZON",
		"CITRIX",
	}[vdipe]
}

func NewVDIProvider(enumString string) VDIProviderEnum {
	return map[string]VDIProviderEnum{
		VDIProviderNone.String():    VDIProviderNone,
		VDIProviderHorizon.String(): VDIProviderHorizon,
		VDIProviderCitrix.String():  VDIProviderCitrix,
	}[strings.ToUpper(enumString)]
}

func NewInfrastructureProvider(enumString string) InfrastructureProviderEnum {
	return map[string]InfrastructureProviderEnum{
		InfrastructureProviderNone.String():  InfrastructureProviderNone,
		InfrastructureProviderAWS.String():   InfrastructureProviderAWS,
		InfrastructureProviderAzure.String(): InfrastructureProviderAzure,
		InfrastructureProviderGCP.String():   InfrastructureProviderGCP,
	}[strings.ToUpper(enumString)]
}

func NewVirtualizationProvider(enumString string) VirtualizationProviderEnum {
	return map[string]VirtualizationProviderEnum{
		VirtualizationProviderNone.String():     VirtualizationProviderNone,
		VirtualizationProviderWorkload.String(): VirtualizationProviderWorkload,
		VirtualizationProviderAWS.String():      VirtualizationProviderAWS,
	}[strings.ToUpper(enumString)]
}

func NewDeploymentType(enumString string) DeploymentTypeEnum {
	return map[string]DeploymentTypeEnum{
		DeploymentTypeEndpoint.String(): DeploymentTypeEndpoint,
		DeploymentTypeWorkload.String(): DeploymentTypeWorkload,
		DeploymentTypeVDI.String():      DeploymentTypeVDI,
		DeploymentTypeAWS.String():      DeploymentTypeAWS,
		DeploymentTypeAzure.String():    DeploymentTypeAzure,
		DeploymentTypeGCP.String():      DeploymentTypeGCP,
	}[strings.ToUpper(enumString)]
}
