package model_test

import (
	"fmt"

	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
	"gitlab.bit9.local/psc/platform/ems/device-service/internal/model"
)

//nolint:lll // long url, sorry
// From https://gitlab.bit9.local/cb-defense/cbd/-/blob/main/backend/domainModel/src/main/java/com/scargo/domainmodel/entity/support/DeploymentType.java
//
//   ENDPOINT(0),
//   WORKLOAD(1),
//   VDI(2),
//   AWS(3),
//   AZURE(4),
//   GCP(5);

var _ = Describe("Deployment Type Enum", func() {
	It("should have the correct values", func() {
		Expect(model.DeploymentTypeEndpoint).To(BeEquivalentTo(0))
		Expect(model.DeploymentTypeWorkload).To(BeEquivalentTo(1))
		Expect(model.DeploymentTypeVDI).To(BeEquivalentTo(2))
		Expect(model.DeploymentTypeAWS).To(BeEquivalentTo(3))
		Expect(model.DeploymentTypeAzure).To(BeEquivalentTo(4))
		Expect(model.DeploymentTypeGCP).To(BeEquivalentTo(5))
	})

	It("should implement stringer interface", func() {
		var _ fmt.Stringer = (*model.DeploymentTypeEnum)(nil)
	})

	It("should return cbd strings from string func", func() {
		Expect(model.DeploymentTypeEndpoint.String()).To(Equal("ENDPOINT"))
		Expect(model.DeploymentTypeWorkload.String()).To(Equal("WORKLOAD"))
		Expect(model.DeploymentTypeVDI.String()).To(Equal("VDI"))
		Expect(model.DeploymentTypeAWS.String()).To(Equal("AWS"))
		Expect(model.DeploymentTypeAzure.String()).To(Equal("AZURE"))
		Expect(model.DeploymentTypeGCP.String()).To(Equal("GCP"))
	})

	It("should return new", func() {
		Expect(model.NewDeploymentType("ENDPOINT")).To(Equal(model.DeploymentTypeEndpoint))
		Expect(model.NewDeploymentType("WORKLOAD")).To(Equal(model.DeploymentTypeWorkload))
		Expect(model.NewDeploymentType("VDI")).To(Equal(model.DeploymentTypeVDI))
		Expect(model.NewDeploymentType("AWS")).To(Equal(model.DeploymentTypeAWS))
		Expect(model.NewDeploymentType("AZURE")).To(Equal(model.DeploymentTypeAzure))
		Expect(model.NewDeploymentType("GCP")).To(Equal(model.DeploymentTypeGCP))
	})
})

//nolint:lll // long url, sorry
// From https://gitlab.bit9.local/cb-defense/cbd/-/blob/main/backend/domainModel/src/main/java/com/scargo/domainmodel/entity/support/SubDeploymentType.java
//
//  VMWARE_VIRTUAL_MACHINE,
//  AWS_VIRTUAL_MACHINE_EC2;

var _ = Describe("Sub Deployment Type Enum", func() {
	It("should implement stringer interface", func() {
		var _ fmt.Stringer = (*model.SubDeploymentTypeEnum)(nil)
	})

	It("should return cbd strings from string func", func() {
		Expect(model.SubDeploymentTypeNone.String()).To(Equal("NONE"))
		Expect(model.SubDeploymentTypeVMwareVirtualMachine.String()).To(Equal("VMWARE_VIRTUAL_MACHINE"))
		Expect(model.SubDeploymentTypeAWSVirtualMachineEC2.String()).To(Equal("AWS_VIRTUAL_MACHINE_EC2"))
	})
})

//nolint:lll // long url, sorry
// From https://gitlab.bit9.local/cb-defense/cbd/-/blob/main/backend/domainModel/src/main/java/com/scargo/domainmodel/entity/support/InfrastructureProvider.java
//
//   AWS("AWS"),
//   AZURE("AZURE"),
//   GCP("GCP"),
//   NONE("NONE");

var _ = Describe("Infrastructure Provider Enum", func() {
	It("should implement stringer interface", func() {
		var _ fmt.Stringer = (*model.InfrastructureProviderEnum)(nil)
	})

	It("should return cbd strings from string func", func() {
		Expect(model.InfrastructureProviderNone.String()).To(Equal("NONE"))
		Expect(model.InfrastructureProviderAWS.String()).To(Equal("AWS"))
		Expect(model.InfrastructureProviderAzure.String()).To(Equal("AZURE"))
		Expect(model.InfrastructureProviderGCP.String()).To(Equal("GCP"))
	})

	It("should return new", func() {
		Expect(model.NewInfrastructureProvider("bad")).To(Equal(model.InfrastructureProviderNone))
		Expect(model.NewInfrastructureProvider("aws")).To(Equal(model.InfrastructureProviderAWS))
		Expect(model.NewInfrastructureProvider("Azure")).To(Equal(model.InfrastructureProviderAzure))
		Expect(model.NewInfrastructureProvider("GCP")).To(Equal(model.InfrastructureProviderGCP))
	})
})

//nolint:lll // long url, sorry
// From https://gitlab.bit9.local/cb-defense/cbd/-/blob/main/backend/backendImpl/src/main/java/com/scargo/backendservices/device/service/impl/DeviceStatusService.java
//
//   private static final String WORKLOAD_VIRTUALIZATION_PROVIDER = "VMW_ESX";
//   private static final String AWS_VIRTUALIZATION_PROVIDER = "AWS-EC2";

//nolint:dupl // similar testing pattern for enum, but not duplicated
var _ = Describe("Virtualization Provider Enum", func() {
	It("should implement stringer interface", func() {
		var _ fmt.Stringer = (*model.VirtualizationProviderEnum)(nil)
	})

	It("should return cbd strings from string func", func() {
		Expect(model.VirtualizationProviderNone.String()).To(Equal("NONE"))
		Expect(model.VirtualizationProviderWorkload.String()).To(Equal("VMW_ESX"))
		Expect(model.VirtualizationProviderAWS.String()).To(Equal("AWS-EC2"))
	})

	It("should return new", func() {
		Expect(model.NewVirtualizationProvider("bad")).To(Equal(model.VirtualizationProviderNone))
		Expect(model.NewVirtualizationProvider("None")).To(Equal(model.VirtualizationProviderNone))
		Expect(model.NewVirtualizationProvider("vmw_esx")).To(Equal(model.VirtualizationProviderWorkload))
		Expect(model.NewVirtualizationProvider("AWS-EC2")).To(Equal(model.VirtualizationProviderAWS))
	})
})

//nolint:lll // long url, sorry
// From https://gitlab.bit9.local/cb-defense/cbd/-/blob/main/backend/domainModel/src/main/java/com/scargo/domainmodel/entity/support/VdiProvider.java
//
//   HORIZON("HORIZON"),
//   CITRIX("CITRIX"),
//   NONE("NONE");

//nolint:dupl // similar testing pattern for enum, but not duplicated
var _ = Describe("VDI Provider Enum", func() {
	It("should implement stringer interface", func() {
		var _ fmt.Stringer = (*model.VDIProviderEnum)(nil)
	})

	It("should return cbd strings from string func", func() {
		Expect(model.VDIProviderNone.String()).To(Equal("NONE"))
		Expect(model.VDIProviderHorizon.String()).To(Equal("HORIZON"))
		Expect(model.VDIProviderCitrix.String()).To(Equal("CITRIX"))
	})

	It("should return new", func() {
		Expect(model.NewVDIProvider("bad")).To(Equal(model.VDIProviderNone))
		Expect(model.NewVDIProvider("None")).To(Equal(model.VDIProviderNone))
		Expect(model.NewVDIProvider("horizon")).To(Equal(model.VDIProviderHorizon))
		Expect(model.NewVDIProvider("CITRIX")).To(Equal(model.VDIProviderCitrix))
	})
})
