package model_test

import (
	"errors"
	"strconv"

	"carbonblack.com/protocols/psc/ems/server/deviceevent"
	"gitlab.bit9.local/psc/platform/ems/device-service/internal/model"

	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
)

var _ = Describe("Device Events Record", func() {
	var (
		rec *model.DeviceEventsRecord
		err error
	)

	device := &model.Device{
		DeploymentType:       model.DeploymentTypeEndpoint,
		ExternallyAssignedID: "0",
		Hostname:             "testHost",
		ID:                   0,
		OrgKey:               "testOrgKey",
		Type:                 0,
	}

	Context("NewDeviceEventsRecord", func() {
		It("should return a device event record", func() {
			rec = model.NewDeviceEventsRecord(device, deviceevent.EmsChangeEvent_UPDATE)

			Expect(rec.Event).ToNot(BeNil())
			Expect(rec.Event.GetDeviceProperties()).ToNot(BeNil())
			Expect(err).To(BeNil())

			Expect(rec.Event.GetDeviceProperties().GetDeploymentType().String()).
				To(Equal(model.DeploymentTypeEndpoint.String()))
			Expect(rec.Event.GetDeviceProperties().GetOsType().String()).
				To(Equal("UNKNOWN"))
			Expect(rec.Event.GetDeviceProperties().GetExternallyAssignedId()).
				To(Equal("0"))

			Expect(rec.Event.GetObjectId()).To(Equal(int64(0)))
			Expect(rec.Event.GetEventType()).To(Equal(deviceevent.EmsChangeEvent_UPDATE))
			Expect(rec.Event.GetObjectType()).To(Equal(deviceevent.EmsChangeEvent_DEVICE))
			Expect(rec.Event.GetOrgKey()).To(Equal("testOrgKey"))
			Expect(rec.Event.GetName()).To(Equal("testHost"))
		})
	})

	Context("Marshal", func() {
		It("should marshal record into bytes", func() {
			rec = model.NewDeviceEventsRecord(device, deviceevent.EmsChangeEvent_UPDATE)
			bytes, err := rec.Marshal()
			Expect(len(bytes)).To(Not(Equal(0)))
			Expect(err).To(BeNil())
		})

		///nolint:govet // testing that error thrown is a specific error, ErrDeviceChangeEventMissing
		It("should return error when record is misformatted", func() {
			rec := &model.DeviceEventsRecord{}
			bytes, err := rec.Marshal()
			Expect(len(bytes)).To(Equal(0))
			Expect(err).ToNot(BeNil())
			Expect(errors.Is(err, model.ErrDeviceChangeEventMissing))
		})
	})

	Context("PartitionKey", func() {
		It("should return records object id", func() {
			rec = model.NewDeviceEventsRecord(device, deviceevent.EmsChangeEvent_UPDATE)
			key, err := rec.PartitionKey()
			Expect(key).To(Equal(strconv.Itoa(int(rec.Event.ObjectId))))
			Expect(err).To(BeNil())
		})

		///nolint:govet // testing that error thrown is a specific error, ErrDeviceChangeEventMissing
		It("should return error when record is misformatted", func() {
			rec := &model.DeviceEventsRecord{}
			key, err := rec.PartitionKey()
			Expect(key).To(Equal(""))
			Expect(errors.Is(err, model.ErrDeviceChangeEventMissing))
		})
	})
})
