package model

import (
	"errors"
	"fmt"
)

type MessageCodeEnum int

var ErrInvalidMsgCode = errors.New("invalid message code")

const (
	DeviceStatusRequestCode MessageCodeEnum = 8193
)

const (
	DeviceMetaDataRequestCode MessageCodeEnum = 8370
)

func (msgCode MessageCodeEnum) String() string {
	switch msgCode {
	case DeviceStatusRequestCode:
		return "8193"
	case DeviceMetaDataRequestCode:
		return "8370"
	default:
		return fmt.Sprintf("%d", int(msgCode))
	}
}

func MessageCode(msgCode string) (MessageCodeEnum, error) {
	switch msgCode {
	case "8193":
		return DeviceStatusRequestCode, nil
	case "8370":
		return DeviceMetaDataRequestCode, nil
	default:
		return 0, fmt.Errorf("could not find message code enum for string %w", ErrInvalidMsgCode)
	}
}
