package service_test

import (
	"context"
	"encoding/base64"
	"encoding/json"

	"gitlab.bit9.local/psc/platform/ems/device-service/internal/config"
	"gitlab.bit9.local/psc/platform/ems/device-service/internal/model"
	"gitlab.bit9.local/psc/platform/ems/device-service/internal/service"
	"gitlab.bit9.local/psc/platform/ems/device-service/test/mocks"

	"github.com/golang/mock/gomock"
	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
	"github.com/spf13/viper"
)

var _ = Describe("Cbd Service", func() {
	var (
		mockCbdClient *mocks.CbdCaller
		cbdService    service.Cbd
		metadata      map[string]string
		host          string
	)

	Context("service enabled", func() {
		BeforeEach(func() {
			mockCbdClient = mocks.NewCbdCaller(ctrl)
			cbdService = service.NewCbdService(mockCbdClient, true)

			host = "localhost"
			headerMap := map[string][]string{
				"whatever": {"test"},
			}

			headers, err := json.Marshal(headerMap)
			Expect(err).To(BeNil())

			metadata = map[string]string{
				model.DataFieldOriginalRequestHost:    host,
				model.DataFieldOriginalRequestHeaders: string(headers),
				model.DataFieldOriginalRequestMsgPack: base64.StdEncoding.EncodeToString([]byte("testmessage")),
			}
		})

		It("should call cbd status", func() {
			metadata[model.DataFieldMessageType] = model.MessageCodeEnum.String(model.DeviceStatusRequestCode)

			mockCbdClient.EXPECT().Call(
				gomock.Any(), host, viper.GetString(config.CbdClientStatusURI), gomock.Any(), gomock.Any()).Return(nil)
			err := cbdService.Update(context.Background(), metadata)
			Expect(err).To(BeNil())
		})

		It("should call cbd metadata", func() {
			metadata[model.DataFieldMessageType] = model.MessageCodeEnum.String(model.DeviceMetaDataRequestCode)

			mockCbdClient.EXPECT().Call(
				gomock.Any(), host, viper.GetString(config.CbdClientMetadataURI), gomock.Any(), gomock.Any()).Return(nil)
			err := cbdService.Update(context.Background(), metadata)
			Expect(err).To(BeNil())
		})

		It("should error if code isn't status or metadata", func() {
			metadata[model.DataFieldMessageType] = "999"

			err := cbdService.Update(context.Background(), metadata)
			Expect(err).ToNot(BeNil())
		})

		Context("service disabled", func() {
			It("shouldn't do anything", func() {
				cbdService = service.NewCbdService(mockCbdClient, false)

				err := cbdService.Update(context.Background(), metadata)
				Expect(err).To(BeNil())
			})
		})
	})
})
