package service_test

import (
	"errors"

	"gitlab.bit9.local/psc/platform/ems/device-service/internal/service"
	"gitlab.bit9.local/psc/platform/ems/device-service/test/mocks"

	"github.com/hashicorp/go-multierror"
	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
)

var (
	ErrRawCantPingDownstream = errors.New("can't ping downstream")
	ErrCantPingDownstream    = multierror.Append(ErrRawCantPingDownstream)
)

var _ = Describe("Readiness Service", func() {
	var rdyChkr service.ReadinessChecker
	var mockReadinessChecker *mocks.ReadinessChecker

	Context("Readiness Service", func() {
		BeforeEach(func() {
			mockReadinessChecker = mocks.NewReadinessChecker(ctrl)
			rdyChkr = service.NewReadinessService(mockReadinessChecker)
		})

		It("should return new service", func() {
			Expect(rdyChkr).ToNot(BeNil())
		})

		Context("Perform readiness check", func() {
			It("should ping downstream dependencies successfully", func() {
				mockReadinessChecker.EXPECT().IsReady().Times(1).Return(nil)
				err := rdyChkr.IsReady()
				Expect(err).To(BeNil())
			})

			It("should ping downstream dependencies and return error", func() {
				mockReadinessChecker.EXPECT().IsReady().Times(1).Return(ErrCantPingDownstream)
				err := rdyChkr.IsReady()
				Expect(err).ToNot(BeNil())
			})
		})
	})
})
