package service

import (
	"github.com/rs/zerolog"
)

// Verify `Service` struct's `Performer` interface compliance.
var _ Performer = (*Service)(nil)

type Service struct{}

// Manager has access on the service to ensure graceful shutdowns and manage service health,
// but no access to service business logic.
type Manager interface {
	FlushAndClose() error
	Name() string
}

type Performer interface {
	Perform(Logger) error
}

type Logger interface {
	Info() *zerolog.Event
	Error() *zerolog.Event
}

func New() *Service {
	return &Service{}
}

func (s *Service) Perform(logger Logger) error {
	logger.Info().Msg("service be doin stuff")

	return nil
}

func (s *Service) FlushAndClose() error {
	return nil
}

func (*Service) Name() string {
	return "a_service"
}
