package telemetry

import (
	"time"

	"github.com/rcrowley/go-metrics"
	"github.com/spf13/viper"
	"github.com/wavefronthq/go-metrics-wavefront/reporting"
	"github.com/wavefronthq/wavefront-sdk-go/senders"
	"gitlab.bit9.local/psc/platform/ems/device-service/internal/config"
)

//nolint:gochecknoglobals // singleton reporter
var MetricReporter reporting.WavefrontMetricsReporter

// SetupMetrics creates a metric registry.
func SetupMetrics(sender senders.Sender) {
	if !viper.GetBool(config.MetricsEnabledKey) || sender == nil {
		sender = NoopSender{}
	}

	MetricReporter = reporting.NewMetricsReporter(
		sender,
		reporting.ApplicationTag(config.GetWFAppTags()),
		reporting.Prefix(viper.GetString(config.MetricsPrefixKey)),
		reporting.RuntimeMetric(viper.GetBool(config.MetricsRuntimeEnabledKey)),
	)
}

func CounterInc(name string, dims map[string]string, count int64) {
	if metric, ok := MetricReporter.GetOrRegisterMetric(
		name,
		metrics.NewCounter,
		dims,
	).(metrics.Counter); ok {
		metric.Inc(count)
	}
}

func HistogramUpdateTimeSince(name string, dims map[string]string, start time.Time) {
	if metric, ok := MetricReporter.GetOrRegisterMetric(
		name,
		reporting.NewHistogram(),
		dims,
	).(metrics.Histogram); ok {
		metric.Update(time.Since(start).Milliseconds())
	}
}

func GaugeUpdate(name string, dims map[string]string, val int64) {
	if metric, ok := MetricReporter.GetOrRegisterMetric(
		name,
		metrics.NewGauge(),
		dims,
	).(metrics.Gauge); ok {
		metric.Update(val)
	}
}
