package telemetry_test

import (
	"gitlab.bit9.local/psc/platform/ems/device-service/internal/telemetry"

	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
	"github.com/wavefronthq/wavefront-sdk-go/histogram"
	"github.com/wavefronthq/wavefront-sdk-go/senders"
)

var _ = Describe("NoopSender", func() {
	var noopSender *telemetry.NoopSender

	BeforeEach(func() {
		noopSender = &telemetry.NoopSender{}
	})

	It("should return nil or 0 for all functions", func() {
		// no expectations since no returns, but calling methods to ensure no failures
		noopSender.Start()
		noopSender.Close()

		Expect(noopSender.Flush()).To(Succeed())
		Expect(noopSender.GetFailureCount()).To(BeZero())
		Expect(noopSender.SendDeltaCounter("", 0, "", map[string]string{})).To(Succeed())
		Expect(noopSender.SendEvent("", 0, 0, "", map[string]string{})).To(Succeed())
		Expect(noopSender.SendMetric("", 0, 0, "", map[string]string{})).To(Succeed())

		err := noopSender.SendDistribution(
			"",
			[]histogram.Centroid{},
			map[histogram.Granularity]bool{},
			0,
			"",
			map[string]string{},
		)
		Expect(err).ShouldNot(HaveOccurred())

		err = noopSender.SendSpan(
			"",
			0,
			0,
			"",
			"",
			"",
			[]string{},
			[]string{},
			[]senders.SpanTag{},
			[]senders.SpanLog{},
		)
		Expect(err).ShouldNot(HaveOccurred())
	})
})
