package telemetry

import (
	"github.com/opentracing/opentracing-go"
	"github.com/rs/zerolog/log"
	"github.com/spf13/viper"
	"github.com/wavefronthq/wavefront-opentracing-sdk-go/reporter"
	"github.com/wavefronthq/wavefront-opentracing-sdk-go/tracer"
	"github.com/wavefronthq/wavefront-sdk-go/senders"
	"gitlab.bit9.local/psc/platform/ems/device-service/internal/config"
	"google.golang.org/grpc/metadata"
)

func SetupTracing(sender senders.Sender) {
	if !viper.GetBool(config.TracingEnabledKey) || sender == nil {
		sender = NoopSender{}
	}

	if viper.GetBool(config.TracingEnabledKey) {
		tracerOptions := []tracer.Option{}

		samplerRate := config.GetWFSamplerRate()
		if samplerRate > 0 {
			tracerOptions = append(tracerOptions, tracer.WithSampler(tracer.RateSampler{Rate: samplerRate}))
		}

		wfReporter := reporter.New(sender, config.GetWFAppTags())
		globalTracer := tracer.New(wfReporter, tracerOptions...)

		opentracing.InitGlobalTracer(globalTracer)

		log.Info().Msg("Tracing enabled")
	}
}

//nolint:ireturn
func SpanContextFromGRPC(grpcMetadata metadata.MD) opentracing.SpanContext {
	carrier := opentracing.HTTPHeadersCarrier{}

	for key, val := range grpcMetadata {
		carrier.Set(key, val[0])
	}

	spanCtx, err := opentracing.GlobalTracer().Extract(opentracing.HTTPHeaders, &carrier)
	if err != nil {
		log.Debug().Msgf("error extracting trace data from context: %v", err)
	}

	return spanCtx
}
