package wavefront

import (
	"errors"
	"fmt"

	"gitlab.bit9.local/psc/platform/ems/device-service/internal/config"

	"github.com/rs/zerolog/log"
	"github.com/spf13/viper"
	"github.com/wavefronthq/wavefront-sdk-go/senders"
)

var ErrUnknownWavefrontMode = errors.New("unknown wavefront mode")

// nolint:funlen
// InitSender initializes a wavefront sender.
func InitSender() senders.Sender { //nolint:ireturn // no access to struct
	var (
		err    error
		sender senders.Sender
	)

	appTags := config.GetWFAppTags()
	wavefrontMode := viper.GetString(config.WFModeKey)

	log.Debug().
		Str("wf_mode", wavefrontMode).
		Str("wf_host", viper.GetString(config.WFHostKey)).
		Str("wf_port", viper.GetString(config.WFPortsMetricsKey)).
		Str("wf_tracing_port", viper.GetString(config.WFPortsTracingKey)).
		Str("wf_flush_interval", viper.GetString(config.WFFlushIntervalSecsKey)).
		Str("wf_tags", fmt.Sprintf("%v", appTags)).
		Msg("Configuring Wavefront sender")

	switch wavefrontMode {
	case "direct":
		if viper.GetString(config.WFTokenKey) == "" {
			log.Error().Msgf("%s cannot be empty, Wavefront integration is disabled", config.WFTokenKey)

			return nil
		}

		sender, err = senders.NewSender(
			fmt.Sprintf("https://%s@vmwareprod.wavefront.com", viper.GetString(config.WFTokenKey)),
			senders.BatchSize(viper.GetInt(config.WFBatchSizeKey)),
			senders.MaxBufferSize(viper.GetInt(config.WFMaxBufferSizeKey)),
			senders.FlushIntervalSeconds(viper.GetInt(config.WFFlushIntervalSecsKey)),
			senders.SDKMetricsTags(appTags.Map()),
		)
		if err != nil {
			log.Err(err).Msg("failed to create direct sender")
		}
	case "proxy":
		sender, err = senders.NewSender(
			fmt.Sprintf("http://%s", viper.GetString(config.WFHostKey)),
			senders.MetricsPort(viper.GetInt(config.WFPortsMetricsKey)),
			senders.TracesPort(viper.GetInt(config.WFPortsTracingKey)),
			senders.FlushIntervalSeconds(viper.GetInt(config.WFFlushIntervalSecsKey)),
			senders.SDKMetricsTags(appTags.Map()),
		)
		if err != nil {
			log.Err(err).Msg("failed to create proxy sender")
		}
	case "disabled":
		log.Info().Msg("Wavefront integration is disabled; no metrics or tracing data will be reported")

		sender = nil
	case "":
		log.Info().Msg("Wavefront is not configured; no metrics or tracing data will be reported")

		sender = nil
	default:
		log.Err(ErrUnknownWavefrontMode).Msgf("mode: %s", wavefrontMode)

		sender = nil
	}

	return sender
}
